/*
ID: espr1t
TASK: Rotating Maze
KEYWORDS: Medium, Extra Parameter DP, Probabilities
*/

#include <cstdio>
#include <cstring>
#include <vector>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

const int MAX = 1024;
const int RETRIES = 100;

int n, m;
int a[MAX][MAX];
vector < vector < vector <double> > > dp;

// Probability to move both ways, when on end row or column
double b1[2][2] = {{0.25, 0.50}, {0.50, 1.00}};
// Probability to move right, when on inner row or column
double r2[2][2][2] = {{{0.25, 0.25}, {0.50, 0.50}}, {{0.375, 0.25}, {0.75, 0.50}}};
// Probability to move down, when on inner row or column
double d2[2][2][2] = {{{0.25, 0.50}, {0.25, 0.50}}, {{0.375, 0.75}, {0.25, 0.50}}};


double go(int row, int col, int rem) {
    if (row == n - 1 && col == m - 1)
        return 0.0;
    if (rem <= 0)
        return n - row - 1 + m - col - 1;
    if (dp[row][col][rem] > -0.5)
        return dp[row][col][rem];
    
    double chanceR = r2[a[row][col]][a[row][col + 1]][a[row + 1][col]];
    double chanceD = d2[a[row][col]][a[row][col + 1]][a[row + 1][col]];
    if (row == n - 1)
        chanceR = b1[a[row][col]][a[row][col + 1]], chanceD = 0.0;
    if (col == m - 1)
        chanceD = b1[a[row][col]][a[row + 1][col]], chanceR = 0.0;
    
    double ans = 1.0;
    if (chanceR > 0.0)
        ans += chanceR * go(row, col + 1, RETRIES - 1);
    if (chanceD > 0.0)
        ans += chanceD * go(row + 1, col, RETRIES - 1);
    if (1.0 - chanceR - chanceD > 0.0)
        ans += (1.0 - chanceR - chanceD) * go(row, col, rem - 1);

    return dp[row][col][rem] = ans;
}

int main(void) {
//    in = fopen("RotatingMaze.in", "rt");
    
    fscanf(in, "%d %d", &n, &m);
    for (int row = 0; row < n; row++) {
        char line[MAX];
        fscanf(in, "%s", line);
        for (int col = 0; col < m; col++)
            a[row][col] = (line[col] == '+');
    }

    dp.resize(n);
    for (int i = 0; i < n; i++)
        dp[i].resize(m, vector <double>(RETRIES, -1));
    fprintf(out, "%.6lf\n", go(0, 0, RETRIES - 1));
    return 0;
}
