/*
ID: espr1t
TASK: Rotating Maze
KEYWORDS: Medium, Monte-Carlo (Slow -> Not Precise Enough)
*/

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

const int MAX = 1024;
const double MAX_EXEC_TIME = 0.42;

int n, m;
char board[MAX][MAX];

double monteCarlo() {
    if (n > 5 || m > 5)
        return -1;
    
    bool canGoDown[4][4] = {
        {0, 0, 0, 0}, {0, 1, 1, 1}, {0, 1, 1, 1}, {0, 1, 1, 1}
    };
    bool canGoRight[4][4] {
        {1, 0, 1, 1}, {0, 0, 0, 0}, {1, 0, 1, 1}, {1, 0, 1, 1}
    };

    unsigned sTime = clock();
    int numIters = 0, totalTime = 0;
    while (true) {
        if (numIters % 10000 == 0) {
            if ((double)(clock() - sTime) / CLOCKS_PER_SEC > MAX_EXEC_TIME) {
                break;
            }
        }
        numIters++;
        
        static int brd[16][16];
        for (int row = 0; row < n; row++) {
            for (int col = 0; col < m; col++)
                brd[row][col] = ((board[row][col] == '-') ? 0 :
                                ((board[row][col] == '|') ? 1 : 2));
        }
        
        int row = 0, col = 0;
        while (row != n - 1 || col != m - 1) {
            totalTime++;

            int rnd = rand();
            int cCell = brd[row + 0][col + 0] ^ !!(rnd & (1 << 3));
            int rCell = brd[row + 0][col + 1] ^ !!(rnd & (1 << 4));
            int dCell = brd[row + 1][col + 0] ^ !!(rnd & (1 << 5));
            
            int addRow = row < n - 1 && canGoDown[cCell][dCell];
            int addCol = col < m - 1 && canGoRight[cCell][rCell];
            row += addRow, col += addCol;
            if (addRow && addCol)
                (rnd & (1 << 6)) ? row-- : col--;
        }
    }
//    fprintf(stderr, "    -- %d iterations done (%d moves)\n", numIters, totalTime);
    return (double)totalTime / numIters;
}

int main(void) {
//    in = fopen("RotatingMaze.in", "rt");
    
    fscanf(in, "%d %d", &n, &m);
    for (int row = 0; row < n; row++)
        fscanf(in, "%s", board[row]);
    
    fprintf(out, "%.2lf0000\n", monteCarlo());
    return 0;
}
