/*
ID: espr1t
TASK: Rotating Maze
KEYWORDS: TestGen & Solver
*/

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>
#include <cmath>
#include <vector>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

const int MAX = 1024;

int n, m;
int a[MAX][MAX];
char board[MAX][MAX];

// Probability to move both ways, when on boundary row or column
double b1[2][2] = {{0.25, 0.50}, {0.50, 1.00}};
// Probability to move right, when not on boundary row or column
double r2[2][2][2] = {{{0.25, 0.25}, {0.50, 0.50}}, {{0.375, 0.25}, {0.75, 0.50}}};
// Probability to move down, when not on boundary row or column
double d2[2][2][2] = {{{0.25, 0.50}, {0.25, 0.50}}, {{0.375, 0.75}, {0.25, 0.50}}};


/**
 * Author's solution
 */

double dyn[MAX][MAX];
double recurse(int row, int col) {
    if (row == n - 1 && col == m - 1)
        return 0.0;
    if (dyn[row][col] == dyn[row][col])
        return dyn[row][col];
    
    double chanceR = r2[a[row][col]][a[row][col + 1]][a[row + 1][col]];
    double chanceD = d2[a[row][col]][a[row][col + 1]][a[row + 1][col]];
    if (row == n - 1)
        chanceR = b1[a[row][col]][a[row][col + 1]], chanceD = 0.0;
    if (col == m - 1)
        chanceD = b1[a[row][col]][a[row + 1][col]], chanceR = 0.0;
    
    double ans = 0.0;
    ans += chanceR > 0.0 ? chanceR * recurse(row, col + 1) : 0.0;
    ans += chanceD > 0.0 ? chanceD * recurse(row + 1, col) : 0.0;
    ans = (1.0 + ans) / (chanceR + chanceD);

    return dyn[row][col] = ans;
}

double eval() {
    memset(dyn, -1, sizeof(dyn));
    return recurse(0, 0);
}

/**
 * DAG by extra parameter
 */

const int RETRIES = 100;
vector < vector < vector <double> > > dp;

double go(int row, int col, int rem) {
    if (row == n - 1 && col == m - 1)
        return 0.0;
    if (rem <= 0)
        return n - row - 1 + m - col - 1;
    if (dp[row][col][rem] > -0.5)
        return dp[row][col][rem];
    
    double chanceR = r2[a[row][col]][a[row][col + 1]][a[row + 1][col]];
    double chanceD = d2[a[row][col]][a[row][col + 1]][a[row + 1][col]];
    if (row == n - 1)
        chanceR = b1[a[row][col]][a[row][col + 1]], chanceD = 0.0;
    if (col == m - 1)
        chanceD = b1[a[row][col]][a[row + 1][col]], chanceR = 0.0;
    
    double ans = 1.0;
    if (chanceR > 0.0)
        ans += chanceR * go(row, col + 1, RETRIES - 1);
    if (chanceD > 0.0)
        ans += chanceD * go(row + 1, col, RETRIES - 1);
    if (1.0 - chanceR - chanceD > 0.0)
        ans += (1.0 - chanceR - chanceD) * go(row, col, rem - 1);

    return dp[row][col][rem] = ans;
}

double expandedDp() {
    // 35000 * 100 * 8 ~= 30MB
//    if (n * m > 35000)
//        return -1;

    dp.clear();
    dp.resize(n);
    for (int i = 0; i < n; i++)
        dp[i].resize(m, vector <double>(RETRIES, -1));
    return go(0, 0, RETRIES - 1);
}

/**
 * Monte-Carlo Approach
 */
const double MAX_EXEC_TIME = 0.42;

double monteCarlo() {
    if (n > 5 || m > 5)
        return -1;
    
    bool canGoDown[4][4] = {
        {0, 0, 0, 0}, {0, 1, 1, 1}, {0, 1, 1, 1}, {0, 1, 1, 1}
    };
    bool canGoRight[4][4] {
        {1, 0, 1, 1}, {0, 0, 0, 0}, {1, 0, 1, 1}, {1, 0, 1, 1}
    };

    unsigned sTime = clock();
    int numIters = 0, totalTime = 0;
    while (true) {
        if (numIters % 10000 == 0) {
            if ((double)(clock() - sTime) / CLOCKS_PER_SEC > MAX_EXEC_TIME) {
                break;
            }
        }
        numIters++;
        
        static int brd[16][16];
        for (int row = 0; row < n; row++) {
            for (int col = 0; col < m; col++)
                brd[row][col] = ((board[row][col] == '-') ? 0 :
                                ((board[row][col] == '|') ? 1 : 2));
        }
        
        int row = 0, col = 0;
        while (row != n - 1 || col != m - 1) {
            totalTime++;

            int rnd = rand();
            int cCell = brd[row + 0][col + 0] ^ !!(rnd & (1 << 3));
            int rCell = brd[row + 0][col + 1] ^ !!(rnd & (1 << 4));
            int dCell = brd[row + 1][col + 0] ^ !!(rnd & (1 << 5));
            
            int addRow = row < n - 1 && canGoDown[cCell][dCell];
            int addCol = col < m - 1 && canGoRight[cCell][rCell];
            row += addRow, col += addCol;
            if (addRow && addCol)
                (rnd & (1 << 6)) ? row-- : col--;
        }
    }
    fprintf(stderr, "    -- %d iterations done (%d moves)\n", numIters, totalTime);
    return (double)totalTime / numIters;
}

double solve(int testNum) {
    unsigned sTime;
    fprintf(stderr, "Solving testcase %d...\n", testNum);
    
    sTime = clock();
    double ans1 = eval();
    fprintf(stderr, "  >> author's answer: %.9lf (%.3lfs)\n",
        ans1, (double)(clock() - sTime) / CLOCKS_PER_SEC);

    sTime = clock();
    double ans2 = expandedDp();
    fprintf(stderr, "  >> expanded DP answer: %.9lf (%.3lfs)\n",
        ans2, (double)(clock() - sTime) / CLOCKS_PER_SEC);

    sTime = clock();
    double ans3 = monteCarlo();
    fprintf(stderr, "  >> monte carlo answer: %.2lf0000000 (%.3lfs)\n",
        ans3, (double)(clock() - sTime) / CLOCKS_PER_SEC);

    if (fabs(ans1 - ans2) < 0.000001)
        fprintf(stderr, "    -- expanded DP solves this test.\n");
    
    return ans1;
}

char bridges[3] = {'-', '|', '+'};

void onlyPluses(int n_, int m_) {
    n = n_, m = m_;
    for (int row = 0; row < n; row++)
        for (int col = 0; col < m; col++)
            board[row][col] = '+';
}

void randomTest(int n_, int m_, int plusHandicap = 3) {
    n = n_, m = m_;
    for (int row = 0; row < n; row++) {
        for (int col = 0; col < m; col++) {
            if (rand() % plusHandicap == 0)
                board[row][col] = '+';
            else {
                board[row][col] = rand() % 2 ? '-' : '|';
            }
        }
    }
}

void exampleTest() {
    n = 2; m = 3;
    board[0][0] = '-'; board[0][1] = '+'; board[0][2] = '|';
    board[1][0] = '+'; board[1][1] = '+'; board[1][2] = '-';
}

int main(void) {
    srand(42);
    int numTests = 20;
    for (int test = 0; test <= numTests; test++) {
        n = m = 0;
        memset(board, 0, sizeof(board));

        if (test == 0) exampleTest();
        else if (test == 1) randomTest(1, 3);
        else if (test == 2) randomTest(3, 1);
        else if (test == 3) randomTest(2, 2);
        else if (test == 4) onlyPluses(3, 3);
        else if (test == 5) randomTest(3, 3);
        else if (test == 6) randomTest(3, 3);
        else if (test == 7) randomTest(1, 50);
        else if (test == 8) randomTest(10, 10);
        else if (test == 9) randomTest(20, 20);
        else if (test == 10) randomTest(30, 30);
        else if (test == 11) randomTest(42, 41);
        else if (test == 12) randomTest(49, 50);
        else if (test == 13) randomTest(72, 91, 13);
        else if (test == 14) randomTest(111, 88, 42);
        else if (test == 15) randomTest(197, 213, 42);
        else if (test == 16) randomTest(333, 571, 5);
        else if (test == 17) randomTest(871, 714, 2);
        else if (test == 18) randomTest(976, 991, 100);
        else if (test == 19) randomTest(1000, 1000, 3);
        else if (test == 20) randomTest(1000, 1000, 1000);

        char inpFile[32], solFile[32];
        sprintf(inpFile, "RotatingMaze.%02d.in", test);
        sprintf(solFile, "RotatingMaze.%02d.sol", test);

        FILE* in = fopen(inpFile, "wt");
        fprintf(in, "%d %d\n", n, m);
        for (int row = 0; row < n; row++)
            fprintf(in, "%s\n", board[row]);
        fclose(in);

        for (int row = 0; row < n; row++)
            for (int col = 0; col < m; col++)
                a[row][col] = (board[row][col] == '+');

        FILE* sol = fopen(solFile, "wt");
        fprintf(sol, "%.6lf\n", solve(test));
        fclose(sol);
    }
    return 0;
}
